@@ -4,8 +4,10 @@ Language tests
 '''
 
 import unittest
+import os
 from weakref import proxy
 from functools import partial
+from textwrap import dedent
 
 
 class BaseClass(object):
@@ -32,7 +34,7 @@ class BaseClass(object):
     def dispatch(self, event_type, *largs, **kwargs):
         pass
 
-    def create_property(self, name, value=None):
+    def create_property(self, name, value=None, default_value=True):
         pass
 
     def is_event_type(self, key):
@@ -75,9 +77,9 @@ class LangTestCase(unittest.TestCase):
         Builder = self.import_builder()
         from kivy.lang import ParserException
         try:
-            Builder.load_string('''#:kivy 1.0
+            Builder.load_string(dedent('''#:kivy 1.0
             <TLangClass>:
-            ''')
+            '''))
             self.fail('Invalid indentation.')
         except ParserException:
             pass
@@ -98,12 +100,12 @@ class LangTestCase(unittest.TestCase):
 
     def test_references(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-<TLangClass>:
-    textinput: textinput
-    TLangClass2:
-        id: textinput
-        ''')
+        Builder.load_string(dedent('''
+        <TLangClass>:
+            textinput: textinput
+            TLangClass2:
+                id: textinput
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
 
@@ -112,16 +114,16 @@ class LangTestCase(unittest.TestCase):
 
     def test_references_with_template(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-[Item@TLangClass3]:
-    title: ctx.title
-<TLangClass>:
-    textinput: textinput
-    Item:
-        title: 'bleh'
-    TLangClass2:
-        id: textinput
-        ''')
+        Builder.load_string(dedent('''
+        [Item@TLangClass3]:
+            title: ctx.title
+        <TLangClass>:
+            textinput: textinput
+            Item:
+                title: 'bleh'
+            TLangClass2:
+                id: textinput
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
 
@@ -130,16 +132,16 @@ class LangTestCase(unittest.TestCase):
 
     def test_references_with_template_case_2(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-[Item@TLangClass3]:
-    title: ctx.title
-<TLangClass>:
-    textinput: textinput
-    TLangClass2:
-        id: textinput
-        Item:
-            title: 'bleh'
-        ''')
+        Builder.load_string(dedent('''
+        [Item@TLangClass3]:
+            title: ctx.title
+        <TLangClass>:
+            textinput: textinput
+            TLangClass2:
+                id: textinput
+                Item:
+                    title: 'bleh'
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
 
@@ -148,18 +150,18 @@ class LangTestCase(unittest.TestCase):
 
     def test_references_with_template_case_3(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-[Item@TLangClass3]:
-    title: ctx.title
-<TLangClass>:
-    textinput: textinput
-    TLangClass2:
-        Item:
-            title: 'bleh'
-        TLangClass2:
+        Builder.load_string(dedent('''
+        [Item@TLangClass3]:
+            title: ctx.title
+        <TLangClass>:
+            textinput: textinput
             TLangClass2:
-                id: textinput
-        ''')
+                Item:
+                    title: 'bleh'
+                TLangClass2:
+                    TLangClass2:
+                        id: textinput
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
 
@@ -168,13 +170,13 @@ class LangTestCase(unittest.TestCase):
 
     def test_with_multiline(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-<TLangClass>:
-    on_press:
-        print('hello world')
-        print('this is working !')
-        self.a = 1
-''')
+        Builder.load_string(dedent('''
+        <TLangClass>:
+            on_press:
+                print('hello world')
+                print('this is working !')
+                self.a = 1
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
         wid.a = 0
@@ -185,13 +187,13 @@ class LangTestCase(unittest.TestCase):
 
     def test_with_eight_spaces(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-<TLangClass>:
-        on_press:
-                print('hello world')
-                print('this is working !')
-                self.a = 1
-''')
+        Builder.load_string(dedent('''
+        <TLangClass>:
+                on_press:
+                        print('hello world')
+                        print('this is working !')
+                        self.a = 1
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
         wid.a = 0
@@ -202,13 +204,13 @@ class LangTestCase(unittest.TestCase):
 
     def test_with_one_space(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-<TLangClass>:
- on_press:
-  print('hello world')
-  print('this is working !')
-  self.a = 1
-''')
+        Builder.load_string(dedent('''
+        <TLangClass>:
+         on_press:
+          print('hello world')
+          print('this is working !')
+          self.a = 1
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
         wid.a = 0
@@ -219,13 +221,13 @@ class LangTestCase(unittest.TestCase):
 
     def test_with_two_spaces(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-<TLangClass>:
-  on_press:
-    print('hello world')
-    print('this is working !')
-    self.a = 1
-''')
+        Builder.load_string(dedent('''
+        <TLangClass>:
+          on_press:
+            print('hello world')
+            print('this is working !')
+            self.a = 1
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
         wid.a = 0
@@ -236,11 +238,11 @@ class LangTestCase(unittest.TestCase):
 
     def test_property_trailingspace(self):
         Builder = self.import_builder()
-        Builder.load_string('''
-<TLangClass>:
-    text : 'original'
-    on_press : self.text = 'changed'
-''')
+        Builder.load_string(dedent('''
+        <TLangClass>:
+            text : 'original'
+            on_press : self.text = 'changed'
+        '''))
         wid = TLangClass()
         Builder.apply(wid)
 
@@ -262,18 +264,18 @@ class LangTestCase(unittest.TestCase):
         class MyWidget(Widget):
             cheese = MyObject()
 
-        Builder.load_string('''
-<MyWidget>:
-    x: 55
-    y: self.width + 10
-    height: self.cheese.value
-    width: 44
+        Builder.load_string(dedent('''
+        <MyWidget>:
+            x: 55
+            y: self.width + 10
+            height: self.cheese.value
+            width: 44
 
-<MySecondWidget@Widget>:
-    x: 55
-    Widget:
-        x: 23
-''')
+        <MySecondWidget@Widget>:
+            x: 55
+            Widget:
+                x: 23
+        '''))
 
         w = MyWidget(x=22, height=12, y=999)
         self.assertEqual(w.x, 22)
@@ -294,6 +296,110 @@ class LangTestCase(unittest.TestCase):
         Builder.apply_rules(wid, 'TLangClassCustom')
         self.assertEqual(wid.obj, 42)
 
+    def test_load_utf8(self):
+        from tempfile import mkstemp
+        from kivy.lang import Builder
+        fd, name = mkstemp()
+        os.write(fd, dedent('''
+        Label:
+            text: 'é 😊'
+        ''').encode('utf8'))
+        root = Builder.load_file(name)
+        assert root.text == 'é 😊'
+        os.close(fd)
+
+    def test_bind_fstring(self):
+        from kivy.lang import Builder
+        label = Builder.load_string(dedent('''
+        <TestLabel@Label>:
+            text: f'{self.pos}|{self.size}'
+        TestLabel:
+        '''))
+        assert label.text == '[0, 0]|[100, 100]'
+        label.pos = 150, 200
+        assert label.text == '[150, 200]|[100, 100]'
+
+    def test_bind_fstring_reference(self):
+        from kivy.lang import Builder
+        root = Builder.load_string(dedent('''
+        FloatLayout:
+            Label:
+                id: original
+                text: 'perfect'
+            Label:
+                id: duplicate
+                text: f'{original.text}'
+        '''))
+        assert root.ids.duplicate.text == 'perfect'
+        root.ids.original.text = 'new text'
+        assert root.ids.duplicate.text == 'new text'
+
+    def test_bind_fstring_expressions(self):
+        from kivy.lang import Builder
+        root = Builder.load_string(dedent('''
+        FloatLayout:
+            Label:
+                id: original
+                text: 'perfect'
+            Label:
+                id: target1
+                text: f"{' '.join(p.upper() for p in original.text)}"
+            Label:
+                id: target2
+                text: f"{''.join(sorted({p.upper() for p in original.text}))}"
+            Label:
+                id: target3
+                text: f"{'odd' if len(original.text) % 2 else 'even'}"
+            Label:
+                id: target4
+                text: f"{original.text[len(original.text) // 2:]}"
+            Label:
+                id: target5
+                text: f"{not len(original.text) % 2}"
+            Label:
+                id: target6
+                text: f"{original.text}" + " some text"
+        '''))
+        assert root.ids.target1.text == 'P E R F E C T'
+        assert root.ids.target2.text == 'CEFPRT'
+        assert root.ids.target3.text == 'odd'
+        assert root.ids.target4.text == 'fect'
+        assert root.ids.target5.text == 'False'
+        assert root.ids.target6.text == 'perfect some text'
+        root.ids.original.text = 'new text'
+        assert root.ids.target1.text == 'N E W   T E X T'
+        assert root.ids.target2.text == ' ENTWX'
+        assert root.ids.target3.text == 'even'
+        assert root.ids.target4.text == 'text'
+        assert root.ids.target5.text == 'True'
+        assert root.ids.target6.text == 'new text some text'
+
+    def test_bind_fstring_expressions_should_not_bind(self):
+        from kivy.lang import Builder
+        root = Builder.load_string(dedent('''
+        FloatLayout:
+            Label:
+                id: original
+                text: 'perfect'
+            Label:
+                id: target1
+                text: f"{' '.join([original.text for _ in range(2)])}"
+            Label:
+                id: target2
+                text: f"{original.text.upper()}"
+            Label:
+                id: target3
+                text: f"{sum(obj.width for obj in root.children)}"
+        '''))
+        assert root.ids.target1.text == ' '
+        assert root.ids.target2.text == ''
+        assert root.ids.target3.text == '400'
+        root.ids.original.text = 'new text'
+        root.ids.original.width = 0
+        assert root.ids.target1.text == ' '
+        assert root.ids.target2.text == ''
+        assert root.ids.target3.text == '400'
+
 
 if __name__ == '__main__':
     unittest.main()
