@@ -2,6 +2,9 @@
 Scatter
 =======
 
+.. image:: images/scatter.gif
+    :align: right
+
 :class:`Scatter` is used to build interactive widgets that can be translated,
 rotated and scaled with two or more fingers on a multitouch system.
 
@@ -12,11 +15,11 @@ translation over the entire children tree without changing any widget
 properties. That specific behavior makes the scatter unique, but there are some
 advantages / constraints that you should consider:
 
-#. The children are positioned relative to the scatter similar to a
-   RelativeLayout (see :mod:`~kivy.uix.relativelayout`). So when dragging the
+#. The children are positioned relative to the scatter similarly to a
+   :mod:`~kivy.uix.relativelayout.RelativeLayout`. So when dragging the
    scatter, the position of the children don't change, only the position of
    the scatter does.
-#. The scatter size has no impact on the size of it's children.
+#. The scatter size has no impact on the size of its children.
 #. If you want to resize the scatter, use scale, not size (read #2). Scale
    transforms both the scatter and its children, but does not change size.
 #. The scatter is not a layout. You must manage the size of the children
@@ -147,9 +150,11 @@ class Scatter(Widget):
             self.do_translation_x, self.do_translation_y = value
         else:
             self.do_translation_x = self.do_translation_y = bool(value)
-    do_translation = AliasProperty(
-        _get_do_translation, _set_do_translation,
-        bind=('do_translation_x', 'do_translation_y'))
+
+    do_translation = AliasProperty(_get_do_translation, _set_do_translation,
+                                   bind=('do_translation_x',
+                                         'do_translation_y'),
+                                   cache=True)
     '''Allow translation on the X or Y axis.
 
     :attr:`do_translation` is an :class:`~kivy.properties.AliasProperty` of
@@ -186,6 +191,9 @@ class Scatter(Widget):
     You can put children outside the bounding box of the scatter and still be
     able to touch them.
 
+    :attr:`do_collide_after_children` is a
+    :class:`~kivy.properties.BooleanProperty` and defaults to False.
+
     .. versionadded:: 1.3.0
     '''
 
@@ -238,8 +246,8 @@ class Scatter(Widget):
             if y > ymax:
                 ymax = y
         return (xmin, ymin), (xmax - xmin, ymax - ymin)
-    bbox = AliasProperty(_get_bbox, None, bind=(
-        'transform', 'width', 'height'))
+
+    bbox = AliasProperty(_get_bbox, bind=('transform', 'width', 'height'))
     '''Bounding box of the widget in parent space::
 
         ((x, y), (w, h))
@@ -259,9 +267,11 @@ class Scatter(Widget):
         r = Matrix().rotate(-radians(angle_change), 0, 0, 1)
         self.apply_transform(r, post_multiply=True,
                              anchor=self.to_local(*self.center))
-    rotation = AliasProperty(_get_rotation, _set_rotation, bind=(
-        'x', 'y', 'transform'))
-    '''Rotation value of the scatter.
+
+    rotation = AliasProperty(_get_rotation, _set_rotation,
+                             bind=('x', 'y', 'transform'))
+    '''Rotation value of the scatter in degrees moving in a counterclockwise
+    direction.
 
     :attr:`rotation` is an :class:`~kivy.properties.AliasProperty` and defaults
     to 0.0.
@@ -273,7 +283,7 @@ class Scatter(Widget):
         scale = p1.distance(p2)
 
         # XXX float calculation are not accurate, and then, scale can be
-        # throwed again even with only the position change. So to
+        # thrown again even with only the position change. So to
         # prevent anything wrong with scale, just avoid to dispatch it
         # if the scale "visually" didn't change. #947
         # Remove this ugly hack when we'll be Python 3 only.
@@ -289,6 +299,7 @@ class Scatter(Widget):
         self.apply_transform(Matrix().scale(rescale, rescale, rescale),
                              post_multiply=True,
                              anchor=self.to_local(*self.center))
+
     scale = AliasProperty(_get_scale, _set_scale, bind=('x', 'y', 'transform'))
     '''Scale value of the scatter.
 
@@ -306,7 +317,8 @@ class Scatter(Widget):
         t = Vector(*center) - self.center
         trans = Matrix().translate(t.x, t.y, 0)
         self.apply_transform(trans)
-    center = AliasProperty(_get_center, _set_center, bind=('bbox', ))
+
+    center = AliasProperty(_get_center, _set_center, bind=('bbox',))
 
     def _get_pos(self):
         return self.bbox[0]
@@ -318,7 +330,8 @@ class Scatter(Widget):
         t = Vector(*pos) - _pos
         trans = Matrix().translate(t.x, t.y, 0)
         self.apply_transform(trans)
-    pos = AliasProperty(_get_pos, _set_pos, bind=('bbox', ))
+
+    pos = AliasProperty(_get_pos, _set_pos, bind=('bbox',))
 
     def _get_x(self):
         return self.bbox[0][0]
@@ -328,7 +341,8 @@ class Scatter(Widget):
             return False
         self.pos = (x, self.y)
         return True
-    x = AliasProperty(_get_x, _set_x, bind=('bbox', ))
+
+    x = AliasProperty(_get_x, _set_x, bind=('bbox',))
 
     def _get_y(self):
         return self.bbox[0][1]
@@ -338,7 +352,8 @@ class Scatter(Widget):
             return False
         self.pos = (self.x, y)
         return True
-    y = AliasProperty(_get_y, _set_y, bind=('bbox', ))
+
+    y = AliasProperty(_get_y, _set_y, bind=('bbox',))
 
     def get_right(self):
         return self.x + self.bbox[1][0]
@@ -346,7 +361,7 @@ class Scatter(Widget):
     def set_right(self, value):
         self.x = value - self.bbox[1][0]
 
-    right = AliasProperty(get_right, set_right, bind=('x', 'width'))
+    right = AliasProperty(get_right, set_right, bind=('x', 'bbox'))
 
     def get_top(self):
         return self.y + self.bbox[1][1]
@@ -354,21 +369,23 @@ class Scatter(Widget):
     def set_top(self, value):
         self.y = value - self.bbox[1][1]
 
-    top = AliasProperty(get_top, set_top, bind=('y', 'height'))
+    top = AliasProperty(get_top, set_top, bind=('y', 'bbox'))
 
     def get_center_x(self):
         return self.x + self.bbox[1][0] / 2.
 
     def set_center_x(self, value):
         self.x = value - self.bbox[1][0] / 2.
-    center_x = AliasProperty(get_center_x, set_center_x, bind=('x', 'width'))
+
+    center_x = AliasProperty(get_center_x, set_center_x, bind=('x', 'bbox'))
 
     def get_center_y(self):
         return self.y + self.bbox[1][1] / 2.
 
     def set_center_y(self, value):
         self.y = value - self.bbox[1][1] / 2.
-    center_y = AliasProperty(get_center_y, set_center_y, bind=('y', 'height'))
+
+    center_y = AliasProperty(get_center_y, set_center_y, bind=('y', 'bbox'))
 
     def __init__(self, **kwargs):
         self._touches = []
@@ -390,9 +407,9 @@ class Scatter(Widget):
         p = self.transform_inv.transform_point(x, y, 0)
         return (p[0], p[1])
 
-    def _apply_transform(self, m):
+    def _apply_transform(self, m, pos=None):
         m = self.transform.multiply(m)
-        return super(Scatter, self)._apply_transform(m)
+        return super(Scatter, self)._apply_transform(m, (0, 0))
 
     def apply_transform(self, trans, post_multiply=False, anchor=(0, 0)):
         '''
@@ -402,7 +419,7 @@ class Scatter(Widget):
 
         :Parameters:
             `trans`: :class:`~kivy.graphics.transformation.Matrix`.
-                Transformation matix to be applied to the scatter widget.
+                Transformation matrix to be applied to the scatter widget.
             `anchor`: tuple, defaults to (0, 0).
                 The point to use as the origin of the transformation
                 (uses local widget space).
@@ -469,6 +486,8 @@ class Scatter(Widget):
             return changed
 
         angle = radians(new_line.angle(old_line)) * self.do_rotation
+        if angle:
+            changed = True
         self.apply_transform(Matrix().rotate(angle, 0, 0, 1), anchor=anchor)
 
         if self.do_scale:
@@ -493,10 +512,19 @@ class Scatter(Widget):
             parent.add_widget(self)
             self.dispatch('on_bring_to_front', touch)
 
+    def on_motion(self, etype, me):
+        if me.type_id in self.motion_filter and 'pos' in me.profile:
+            me.push()
+            me.apply_transform_2d(self.to_local)
+            ret = super().on_motion(etype, me)
+            me.pop()
+            return ret
+        return super().on_motion(etype, me)
+
     def on_touch_down(self, touch):
         x, y = touch.x, touch.y
 
-        # if the touch isnt on the widget we do nothing
+        # if the touch isn't on the widget we do nothing
         if not self.do_collide_after_children:
             if not self.collide_point(x, y):
                 return False
@@ -505,9 +533,6 @@ class Scatter(Widget):
         touch.push()
         touch.apply_transform_2d(self.to_local)
         if super(Scatter, self).on_touch_down(touch):
-            # ensure children don't have to do it themselves
-            if 'multitouch_sim' in touch.profile:
-                touch.multitouch_sim = True
             touch.pop()
             self._bring_to_front(touch)
             return True
@@ -559,12 +584,13 @@ class Scatter(Widget):
     def on_transform_with_touch(self, touch):
         '''
         Called when a touch event has transformed the scatter widget.
-        By default this does nothing, but can be overriden by derived
+        By default this does nothing, but can be overridden by derived
         classes that need to react to transformations caused by user
         input.
 
         :Parameters:
-            `touch`: the touch object which triggered the transformation.
+            `touch`:
+                The touch object which triggered the transformation.
 
         .. versionadded:: 1.8.0
         '''
@@ -576,7 +602,8 @@ class Scatter(Widget):
         front of the parent (only if :attr:`auto_bring_to_front` is True)
 
         :Parameters:
-            `touch`: the touch object which brought the scatter to front.
+            `touch`:
+                The touch object which brought the scatter to front.
 
         .. versionadded:: 1.9.0
         '''
@@ -584,7 +611,7 @@ class Scatter(Widget):
 
     def on_touch_up(self, touch):
         x, y = touch.x, touch.y
-        # if the touch isnt on the widget we do nothing, just try children
+        # if the touch isn't on the widget we do nothing, just try children
         if not touch.grab_current == self:
             touch.push()
             touch.apply_transform_2d(self.to_local)
@@ -610,7 +637,8 @@ class ScatterPlane(Scatter):
     '''
 
     def __init__(self, **kwargs):
-        kwargs.setdefault('auto_bring_to_front', False)
+        if 'auto_bring_to_front' not in kwargs:
+            self.auto_bring_to_front = False
         super(ScatterPlane, self).__init__(**kwargs)
 
     def collide_point(self, x, y):
